#!/usr/bin/env ruby

require "gtk"
require "numru/netcdf"
require "numru/nispack"

include NumRu



class Variables
  def initialize; @att=Hash.new; end
  def vobj=(obj); @vobj=obj; end
  def dname=(name); @dname=name; end
  def vobj; @vobj; end
  def dname; @dname; end
  def set_att(name,val); @att[name]=val; end
  def att; @att; end
end

class Dimensions
  def initialize; @att = Hash.new; end
  def value=(value); @value=value; end
  def select=(select); @select=select; end
  def value; @value; end
  def set_att(name,val); @att[name]=val; end
  def att; @att; end
  def select; @select; end
  def select_value; @value[@select.index]; end
end


# draw tyep constant
LINE = 1
CONTOUR = 2
MAP = 4


def s2f(str)
  if str=="" then
    false
  else
    str.to_f
  end
end
def min(x,y)
  if x<y then
    x
  else
    y
  end
end


class GDCL

  def initialize

    @vars = Hash.new
    @dims = Hash.new

    accel = Gtk::AccelGroup.new

# FileSelection
    fs = Gtk::FileSelection.new("")
    fs.ok_button.signal_connect("clicked"){
      fs.hide
      @filecall.call(fs.get_filename)
    }
    fs.cancel_button.signal_connect("clicked"){fs.hide}

# sub windows
    var_window = Gtk::Window.new(Gtk::WINDOW_TOPLEVEL).set_title("Variable")
    axis_window = Gtk::Window.new(Gtk::WINDOW_TOPLEVEL).set_title("Axis")
    @dims_window = Gtk::Window.new(Gtk::WINDOW_TOPLEVEL).set_title("Dimensions")
    attr_window = Gtk::Window.new(Gtk::WINDOW_TOPLEVEL).set_title("Attributes")
    movie_window = Gtk::Window.new(Gtk::WINDOW_TOPLEVEL).set_title("Movie")
    map_window = Gtk::Window.new(Gtk::WINDOW_TOPLEVEL).set_title("Map")
    level_window = Gtk::Window.new(Gtk::WINDOW_TOPLEVEL).set_title("Levels for Contour and Tone")
    var_window.signal_connect("delete_event"){true}
    axis_window.signal_connect("delete_event"){true}
    @dims_window.signal_connect("delete_event"){true}
    attr_window.signal_connect("delete_event"){true}
    movie_window.signal_connect("delete_event"){true}
    map_window.signal_connect("delete_event"){true}
    level_window.signal_connect("delete_event"){true}

# Variable window
    @title_entry = Gtk::Entry.new
    @miss_entry = Gtk::Entry.new
    table = Gtk::Table.new(2,2,false)
    table.attach(Gtk::Label.new("title").show,0,1,0,1)
    table.attach(@title_entry.show,1,2,0,1)
    table.attach(Gtk::Label.new("missing value").show,0,1,1,2)
    table.attach(@miss_entry.show,1,2,1,2)
    var_window.add(table.show)

# Axis window
    @logxbutton = Gtk::CheckButton.new("log scale")
    @logxbutton.signal_connect("clicked"){|w| @draw_data.axis_x.log = w.active?}
    @x_title = Gtk::Entry.new
    @x_title.signal_connect("activate"){|w| @draw_data.axis_x.title = w.get_text}
    @x_unit = Gtk::Entry.new
    @x_unit.signal_connect("activate"){|w| @draw_data.axis_x.unit = w.get_text}
    @x_min = Gtk::Entry.new
    @x_min.signal_connect("activate"){|w|
      str = w.get_text
      if str=="" then
	@draw_data.axis_x.min = false
      else
	@draw_data.axis_x.min = str.to_f
      end
    }
    @x_max = Gtk::Entry.new
    @x_max.signal_connect("activate"){|w|
      str = w.get_text
      if str=="" then
	@draw_data.axis_x.max = false
      else
	@draw_data.axis_x.max = str.to_f
      end
    }
    table = Gtk::Table.new(2,5,false)
    table.attach(@logxbutton.show,1,2,0,1)
    table.attach(Gtk::Label.new("title").show,0,1,1,2)
    table.attach(@x_title.show,1,2,1,2)
    table.attach(Gtk::Label.new("unit").show,0,1,2,3)
    table.attach(@x_unit.show,1,2,2,3)
    table.attach(Gtk::Label.new("min").show,0,1,3,4)
    table.attach(@x_min.show,1,2,3,4)
    table.attach(Gtk::Label.new("max").show,0,1,4,5)
    table.attach(@x_max.show,1,2,4,5)
    x_frame = Gtk::Frame.new("X-Axis")
    x_frame.add(table.show)

    @logybutton = Gtk::CheckButton.new("log scale")
    @logybutton.signal_connect("clicked"){|w| @draw_data.axis_y.log = w.active?}
    @y_title = Gtk::Entry.new
    @y_title.signal_connect("activate"){|w| @draw_data.axis_y.title = w.get_text}
    @y_unit = Gtk::Entry.new
    @y_unit.signal_connect("activate"){|w| @draw_data.axis_y.unit = w.get_text}
    @y_min = Gtk::Entry.new
    @y_min.signal_connect("activate"){|w|
      str = w.get_text
      if str=="" then
	@draw_data.axis_y.min = false
      else
	@draw_data.axis_y.min = str.to_f
      end
    }
    @y_max = Gtk::Entry.new
    @y_max.signal_connect("activate"){|w|
      str = w.get_text
      if str=="" then
	@draw_data.axis_y.max = false
      else
	@draw_data.axis_y.max = str.to_f
      end
    }
    table = Gtk::Table.new(2,5,false)
    table.attach(@logybutton.show,1,2,0,1)
    table.attach(Gtk::Label.new("title").show,0,1,1,2)
    table.attach(@y_title.show,1,2,1,2)
    table.attach(Gtk::Label.new("unit").show,0,1,2,3)
    table.attach(@y_unit.show,1,2,2,3)
    table.attach(Gtk::Label.new("min").show,0,1,3,4)
    table.attach(@y_min.show,1,2,3,4)
    table.attach(Gtk::Label.new("max").show,0,1,4,5)
    table.attach(@y_max.show,1,2,4,5)
    y_frame = Gtk::Frame.new("Y-Axis")
    y_frame.add(table.show)
    axisbox = Gtk::VBox.new
    axisbox.pack_start(x_frame.show)
    axisbox.pack_start(y_frame.show)
    axis_window.add(axisbox.show)

# Attributes window
    @attr_note = Gtk::Notebook::new()
    @attr_note.set_tab_pos(Gtk::POS_TOP)
    box = Gtk::VBox.new(false,0)
    box.pack_start(@attr_note.show)
    box.show
    attr_window.add(box)

# Movie winsow
    start_xpm = ["11 11 2 1",  "  c None", "+  c black",
      "+          ",
      "+++        ",
      "+++++      ",
      "+++++++    ",
      "+++++++++  ",
      "+++++++++++",
      "+++++++++  ",
      "+++++++    ",
      "+++++      ",
      "+++        ",
      "+          "]
    stop_xpm = ["11 11 2 1", "   c None", "+  c black",
      "           ",
      " +++++++++ ",
      " +++++++++ ",
      " +++++++++ ",
      " +++++++++ ",
      " +++++++++ ",
      " +++++++++ ",
      " +++++++++ ",
      " +++++++++ ",
      " +++++++++ ",
      "           "]
    rec_xpm = ["11 11 2 1", "   c None", "+  c red",
      "    +++    ",
      "  +++++++  ",
      " +++++++++ ",
  " +++++++++ ",
      "+++++++++++",
      "+++++++++++",
      "+++++++++++",
      " +++++++++ ",
      " +++++++++ ",
      "  +++++++  ",
      "    +++    "]
    @movie = false
    @movie_step = 1.0
    @rec = false
    @fnum = "001"
    movie_window.realize
    pix, mask = Gdk::Pixmap.create_from_xpm_d(movie_window.window, nil, start_xpm) 
    start_pixmap =  Gtk::Pixmap.new(pix, mask)
    pix, mask = Gdk::Pixmap.create_from_xpm_d(movie_window.window, nil, stop_xpm) 
    stop_pixmap =  Gtk::Pixmap.new(pix, mask)
    pix, mask = Gdk::Pixmap.create_from_xpm_d(movie_window.window, nil, rec_xpm) 
    rec_pixmap =  Gtk::Pixmap.new(pix, mask)
    start_button = Gtk::ToggleButton.new
    start_button.add(Gtk::HBox.new.pack_start(start_pixmap))
    stop_button = Gtk::Button.new
    stop_button.add(Gtk::HBox.new.pack_start(stop_pixmap))
    rec_button = Gtk::ToggleButton.new
    rec_button.add(Gtk::HBox.new.pack_start(rec_pixmap))
    start_button.signal_connect("clicked"){|w|
      if w.active? then
	@time = Time.now
	@movie = Gtk.idle_add{
	  if Time.now - @time > @movie_step then
	    draw_next
	    if @rec then
	      @draw.save_image("dcl"+@fnum+".ppm")
	      @fnum.next!
	    end
	    @time = Time.now
	  end
	  true
	}
      else
	Gtk.idle_remove(@movie)
      end
    }
    stop_button.signal_connect("clicked"){
      if start_button.active? then
	start_button.set_active(false)
	Gtk.idle_remove(@movie)
      end
    }
    rec_button.signal_connect("clicked"){|w|
      @rec = w.active?
    }
    @movie_combo = Gtk::Combo.new.set_usize(50,30)
    @movie_combo.entry.set_editable(false)
    @movie_combo.entry.set_text("-")
    box1 = Gtk::HBox.new(false,0)
    box1.pack_start(@movie_combo)
    box1.pack_start(start_button)
    box1.pack_start(stop_button)
    box1.pack_start(rec_button)
    adjustment = Gtk::Adjustment.new(@movie_step,0.0,3,0.1,1,1)
    adjustment.signal_connect("value_changed"){|w|
      @movie_step=w.value
    }
    scale = Gtk::HScale.new(adjustment)
    scale.set_usize(150,30)
    scale.set_update_policy(Gtk::UPDATE_DELAYED)
    scale.set_draw_value(false)
    box2 = Gtk::HBox.new(false,0)
    box2.pack_start(Gtk::Label.new("fast"))
    box2.pack_start(scale)
    box2.pack_start(Gtk::Label.new("slow"))
    @movie_box = Gtk::VBox.new(false,0)
    @movie_box.pack_start(box1)
    @movie_box.pack_start(box2)
    movie_window.add(@movie_box.show_all)

# Map window
    @maps =  ["ORTHOGRAPHIC","POLAR STEREO","AZIMUTHAL","AZIMUTHAL EQ. AREA"]
    @map_combo = Gtk::Combo.new
    @map_combo.entry.set_editable(false)
    @map_combo.set_popdown_strings(@maps)
    box = Gtk::VBox.new(false,0)
    box.pack_start(@map_combo.show)
    box.show
    map_window.add(box)

# Level window
    @cont_min_entry = Gtk::Entry.new
    @cont_interval_entry = Gtk::Entry.new
    cont_manual_box = Gtk::HBox.new.set_sensitive(false)
    cont_manual_box.pack_start(Gtk::Label.new("min"))
    cont_manual_box.pack_start(@cont_min_entry)
    cont_manual_box.pack_start(Gtk::Label.new("interval"))
    cont_manual_box.pack_start(@cont_interval_entry)
    cont_auto_radio = Gtk::RadioButton.new("auto")
    cont_manual_radio = Gtk::RadioButton.new(cont_auto_radio,"manual")
    cont_auto_radio.signal_connect("toggled"){|w|
      cont_manual_box.set_sensitive(false)
      @cont_level = false
    }
    cont_manual_radio.signal_connect("toggled"){|w|
      cont_manual_box.set_sensitive(true)
      @cont_level = true
    }
    box = Gtk::HBox.new
    box.pack_start(cont_auto_radio)
    box.pack_start(cont_manual_radio)
    box.pack_start(cont_manual_box)
    cont_box = Gtk::VBox.new
    cont_box.pack_start(Gtk::Label.new("Contour"))
    cont_box.pack_start(box)
    @tone_min_entry = Gtk::Entry.new
    @tone_interval_entry = Gtk::Entry.new
    tone_manual_box = Gtk::HBox.new.set_sensitive(false)
    tone_manual_box.pack_start(Gtk::Label.new("min"))
    tone_manual_box.pack_start(@tone_min_entry)
    tone_manual_box.pack_start(Gtk::Label.new("interval"))
    tone_manual_box.pack_start(@tone_interval_entry)
    tone_auto_radio = Gtk::RadioButton.new("auto")
    tone_manual_radio = Gtk::RadioButton.new(tone_auto_radio,"manual")
    tone_auto_radio.signal_connect("toggled"){|w|
      tone_manual_box.set_sensitive(false)
      @tone_level = false
    }
    tone_manual_radio.signal_connect("toggled"){|w|
      tone_manual_box.set_sensitive(true)
      @tone_level = true
    }
    box = Gtk::HBox.new
    box.pack_start(tone_auto_radio)
    box.pack_start(tone_manual_radio)
    box.pack_start(tone_manual_box)
    tone_box = Gtk::VBox.new
    tone_box.pack_start(Gtk::Label.new("Tone"))
    tone_box.pack_start(box)
    level_box = Gtk::VBox.new
    level_box.pack_start(cont_box)
    level_box.pack_start(Gtk::HSeparator.new)
    level_box.pack_start(tone_box)
    level_box.show_all
    level_window.add(level_box)
      

# Variable Combo
    @var_select = Var_select.new("variable")
    @axisx_select = Var_select.new("x-axis")
    @axisy_select = Var_select.new("y-axis")
    @var_select.signal{set_var}
    @axisx_select.signal{set_axisx}
    @axisy_select.signal{set_axisy}

# figure type item
    type_itemfact = Gtk::ItemFactory.new(Gtk::ItemFactory::TYPE_OPTION_MENU, "<type>", accel)
    type_itemfact.create_item("/Line", nil, Gtk::ItemFactory::ITEM){
      @draw_data.type = LINE
      clear_axisx
      @axisx_select.set_text(@vars[@draw_data.var.name].dname[0])
      set_axisx
      @axisx_select.active
    }
    type_itemfact.create_item("/Contour", nil, Gtk::ItemFactory::ITEM){
      @draw_data.type = CONTOUR
      clear_axisx
      @axisx_select.set_text(@vars[@draw_data.var.name].dname[0])
      set_axisx
      @axisx_select.active
    }
    type_itemfact.create_item("/Map", nil, Gtk::ItemFactory::ITEM){
      @draw_data.type = MAP
      clear_axisx
      @vars[@draw_data.var.name].dname.each{|dim|
	if dim.downcase=~/^(lon|x)/ then
	  @axisx_select.set_text(dim)
	  break
	end
      }
      set_axisx
      @axisx_select.active
    }
    @typeomenu = type_itemfact.get_widget("<type>").show
    @linemenu = type_itemfact.get_item("/Line")
    @contourmenu = type_itemfact.get_item("/Contour")

# Draw button
    @drawbutton = Gtk::Button.new("draw").show.set_sensitive(false)
    @drawbutton.signal_connect("clicked"){ draw_x }

# Select table
    table = Gtk::Table.new(2,6,false)
    table.attach(Gtk::Label.new("Variable").show,0,1,0,1)
    table.attach(@var_select.combo,1,2,0,1)
    table.attach(Gtk::Label.new("Figure type").show,0,1,1,2)
    table.attach(@typeomenu,1,2,1,2)
    table.attach(Gtk::Label.new("Axis(x)").show,0,1,2,3)
    table.attach(@axisx_select.combo,1,2,2,3)
    table.attach(Gtk::Label.new("Axis(y)").show,0,1,3,4)
    table.attach(@axisy_select.combo,1,2,3,4)
    table.attach(@drawbutton,1,2,5,6)
    table.show

# Main Menu
    main_itemfact = Gtk::ItemFactory.new(Gtk::ItemFactory::TYPE_MENU_BAR, "<main>", accel)
    main_itemfact.create_item("/File(_F)", nil, Gtk::ItemFactory::BRANCH)
    main_itemfact.create_item("/File(_F)/Open(_O)", nil, Gtk::ItemFactory::ITEM){
      fs.set_title("Open File")
      @filecall = proc{|filename| file_open(filename)}
      fs.show
    }
    main_itemfact.create_item("/File(_F)/Save Image(_S)", "<control>S", Gtk::ItemFactory::ITEM){
      if !@draw.nil? then
	fs.set_title("Save Image")
	@filecall = proc{|filename|
	  if !FileTest.exist?(filename)
	    @draw.save_image(filename)
	  else
	    dialog = Gtk::Dialog.new
	    dialog.set_modal(true)
	    dialog.vbox.pack_start(Gtk::Label.new("Can "+filename+" overwrite ?").show)
	    ok_button=Gtk::Button.new("OK").show
	    ok_button.signal_connect("clicked"){dialog.destroy; @draw.save_image(filename)}
	    cancel_button=Gtk::Button.new("Cancel").show
	    cancel_button.signal_connect("clicked"){dialog.destroy}
	    dialog.action_area.pack_start(ok_button)
	    dialog.action_area.pack_start(cancel_button)
	    dialog.show
	  end
	}
	fs.show
      end
    }
    main_itemfact.create_item("/File(_F)/Save Code(_C)", "<control>C", Gtk::ItemFactory::ITEM){
      if !@draw.nil? then
	fs.set_title("Save Code")
	@filecall = proc{|filename|
	  if !FileTest.exist?(filename)
	    @draw.save_code(filename)
	  else
	    dialog = Gtk::Dialog.new
	    dialog.set_modal(true)
	    dialog.vbox.pack_start(Gtk::Label.new("Can "+filename+" overwrite ?").show)
	    ok_button=Gtk::Button.new("OK").show
	    ok_button.signal_connect("clicked"){dialog.destroy; @draw.save_code(filename)}
	    cancel_button=Gtk::Button.new("Cancel").show
	    cancel_button.signal_connect("clicked"){dialog.destroy}
	    dialog.action_area.pack_start(ok_button)
	    dialog.action_area.pack_start(cancel_button)
	    dialog.show
	  end
	}
	fs.show
      end
    }
    #main_itemfact.create_item("/File(_F)/Close(_C)", "<control>C", Gtk::ItemFactory::ITEM){file_close}
    main_itemfact.create_item("/File(_F)/Quit(_Q)", "<control>Q", Gtk::ItemFactory::ITEM){finish}
    main_itemfact.create_item("/Window(_W)", nil, Gtk::ItemFactory::BRANCH)
    main_itemfact.create_item("/Window(_W)/_Variable", nil, Gtk::ItemFactory::CHECK_ITEM){|w|
      if w.active? then var_window.show else var_window.hide end
    }
    main_itemfact.create_item("/Window(_W)/_Axis", nil, Gtk::ItemFactory::CHECK_ITEM){|w|
      if w.active? then axis_window.show else axis_window.hide end
    }
    main_itemfact.create_item("/Window(_W)/_Dimensions", nil, Gtk::ItemFactory::CHECK_ITEM){|w|
      if w.active? then @dims_window.show else @dims_window.hide end
    }
    main_itemfact.create_item("/Window(_W)/_Attributes", nil, Gtk::ItemFactory::CHECK_ITEM){|w|
      if w.active? then attr_window.show else attr_window.hide end
    }
    main_itemfact.create_item("/Window(_W)/_Movie", nil, Gtk::ItemFactory::CHECK_ITEM){|w|
      if w.active? then movie_window.show else movie_window.hide end
    }
    main_itemfact.create_item("/Window(_W)/_Map", nil, Gtk::ItemFactory::CHECK_ITEM){|w|
      if w.active? then map_window.show else map_window.hide end
    }
    main_itemfact.create_item("/Window(_W)/_Level", nil, Gtk::ItemFactory::CHECK_ITEM){|w|
      if w.active? then level_window.show else level_window.hide end
    }
    main_itemfact.create_item("/Help(_H)", nil, Gtk::ItemFactory::LAST_BRANCH)
    main_itemfact.create_item("/Help(_H)/_About", nil, Gtk::ItemFactory::ITEM){
      dialog = Gtk::Dialog.new.set_title("About this program")
      dialog.vbox.pack_start(Gtk::Label.new("This program is made by Seiya Nishizawa").show)
      dialog.vbox.pack_start(Gtk::Label.new("seiya@kugi.kyoto-u.ac.jp").show)
      ok_button=Gtk::Button.new("OK").show
      ok_button.signal_connect("clicked"){dialog.destroy}
      dialog.action_area.pack_start(ok_button)
      dialog.show
    }
    #main_itemfact.get_widget("/File(F)/Save Image(S)").set_sensitive(false)

# main window
    totalbox = Gtk::VBox.new(false,0)
    totalbox.pack_start(main_itemfact.get_widget("<main>").show,false,true,0)
    totalbox.pack_start(table,false,true,0)
    totalbox.show
    totalbox.signal_connect("destroy"){finish}
    main_window = Gtk::Window.new(Gtk::WINDOW_TOPLEVEL)
    accel.attach(main_window)
    main_window.set_title("gdcl")
    main_window.add(totalbox)
    main_window.signal_connect("delete_event"){finish}
    main_window.signal_connect("destroy"){finish}
    main_window.show

  end


# OpenFile
  def file_open(filename)
    filename = File.expand_path(filename)
    print "open ",filename,"\n"
    begin
      @file = NetCDF.open(filename,mode="r")
    rescue
      dialog = Gtk::Dialog.new.set_title("Error Message")
      dialog.set_modal(true)
      dialog.vbox.pack_start(Gtk::Label.new(filename).show)
      dialog.vbox.pack_start(Gtk::Label.new($!).show)
      ok_button=Gtk::Button.new("OK").show
      ok_button.signal_connect("clicked"){dialog.destroy}
      dialog.action_area.pack_start(ok_button)
      dialog.show
    else
      @draw_data = Draw_data.new
      @draw_data.file_name = filename
      
      clear_var
      @vars.clear
      @dims.clear
      @dimtable.hide if !@dimtable.nil?

      @file.var_names.each_index{|i|
	vn = @file.var_names[i]
	var = @file.var(vn)
	dims = var.dim_names
	if (dims.length==1) && (dims[0]==vn) && (! @dims.has_key?(vn)) then
	  @dims[vn] = Dimensions.new
	  @dims[vn].value = var.get
	  @dims[vn].select = Select_value.new(var.get)
	  var.each_att{|att| @dims[vn].set_att(att.name,att.get)}
	elsif ! @vars.has_key?(vn)
	  @vars[vn] = Variables.new
	  @vars[vn].vobj = var
	  @vars[vn].dname = dims
	  var.each_att{|att| @vars[vn].set_att(att.name,att.get)}
	end
	clist = Gtk::CList.new(["name","value"])
	clist.set_usize(300, 200)
	clist.set_column_width(0,100)
	var.each_att{|att|
	  val = att.get
	  if val.type==NArray then
	    val = att.get.to_a.join(",")
	  end
	  clist.append([att.name,val])
	}
	@attr_note.append_page_menu(clist.show,Gtk::Label.new(vn).show,Gtk::Label.new(vn))
      }
      clist = Gtk::CList.new(["name","value"])
      clist.set_usize(300, 200)
      clist.set_column_width(0,100)
      @file.each_att{|att|
	val = att.get
	if val.type==NArray then
	  val = att.get.to_a.join(",")
	end
	clist.append([att.name,val])
      }
      @attr_note.append_page_menu(clist.show,Gtk::Label.new("global").show,Gtk::Label.new("global").show)

      @dimtable = Gtk::Table.new(2, @dims.length, false)
      i=0
      @dims.each_key{|dim|
	@dimtable.attach(Gtk::Label.new(dim).set_usize(30,20).show, 0, 1, i, i+1 )
	@dimtable.attach(@dims[dim].select.box, 1, 2, i, i+1 )
	i += 1
      }
      @dims_window.add(@dimtable.show)

      @var_select.set(@vars.keys)
      @var_select.active
      set_var
    end

  end

  def set_var
    @draw_data.var.name = @var_select.get
    if @draw_data.var.name =~ /^select/ then
      @draw_data.var.name = nil
    else
      print "variable name is ",@draw_data.var.name,"\n"
      @vars[@draw_data.var.name].att.each{|key,value|
	if key=~/miss/ then @miss_entry.set_text(value[0].to_s) end
	if key=~/name/ then @title_entry.set_text(value) end
	if key=~/unit/ then
	  tmp=@title_entry.get_text+"("+value+")"
	  @title_entry.set_text(tmp)
	end
      }
      ax = @vars[@draw_data.var.name].dname
      clear_axisx
      if ax.length == 1 then
	@draw_data.axis_x.name = ax[0]
	@draw_data.axis_x.value = @dims[@draw_data.axis_x.name].value
	@typeomenu.set_sensitive(false)
	@typeomenu.set_history(0)
	@linemenu.activate
	set_draw
      else
	@typeomenu.set_history(1)
	@contourmenu.activate
	@typeomenu.set_sensitive(true)
	@axisx_select.set(ax)
	@axisx_select.active
	set_axisx
      end
    end
  end

  def set_axisx
    @draw_data.axis_x.name = @axisx_select.get
    if @draw_data.axis_x.name =~ /^select/ then
      @draw_data.axis_x.clear
    else
      print "Axis(x) name is ",@draw_data.axis_x.name,"\n"
      @draw_data.axis_x.value = @dims[@draw_data.axis_x.name].value
      @draw_data.axis_x.title = @draw_data.axis_x.name; @x_title.set_text(@draw_data.axis_x.title)
      @dims[@draw_data.axis_x.name].att.each{|key,value|
	if key=~/unit/ then @draw_data.axis_x.unit=value; @x_unit.set_text(@draw_data.axis_x.unit) end
      }

      clear_axisy
      ay = @vars[@draw_data.var.name].dname.collect{|i| i}
      ay.delete(@draw_data.axis_x.name)
      if @draw_data.type == LINE then
	set_draw
      else
	@axisy_select.set(ay)
	if @draw_data.type == MAP then
	  ay.each{|dim|
	    if dim.downcase=~/^(lat|y)/ then
	      @axisy_select.set_text(dim)
	      break
	    end
	  }
	end
	@axisy_select.active
	set_axisy
      end
    end
  end
  def set_axisy
    @draw_data.axis_y.name = @axisy_select.get
    if @draw_data.axis_y.name =~ /^select/ then
      @draw_data.axis_y.clear
    else
      print "Axis(y) name is ",@draw_data.axis_y.name,"\n"
      @draw_data.axis_y.value = @dims[@draw_data.axis_y.name].value
      @draw_data.axis_y.title = @draw_data.axis_y.name; @y_title.set_text(@draw_data.axis_y.title)
      @dims[@draw_data.axis_y.name].att.each{|key,value|
	if key=~/unit/ then @draw_data.axis_y.unit=value; @y_unit.set_text(@draw_data.axis_y.unit) end
      }

      set_draw
    end
  end

  def set_draw
    dims = @vars[@draw_data.var.name].dname.collect{|i| i}
    dims.delete(@draw_data.axis_x.name)
    dims.delete(@draw_data.axis_y.name)
    clear_draw
    @draw_data.dims.clear
    if dims.length != 0 then
      dims.each{|dim|
	@dims[dim].select.active
	@draw_data.set_dim(dim,@dims[dim].select_value)
      }
      @movie_combo.set_popdown_strings(dims)
      @movie_box.set_sensitive(true)
    end
    @drawbutton.set_sensitive(true)
  end

  def clear_var
    @draw_data.var.clear
    clear_axisx
    @var_select.not_active
  end
  def clear_axisx
    @draw_data.axis_x.clear
    clear_axisy
    @axisx_select.not_active
    @logxbutton.set_active(false)
    @x_title.set_text("")
  end
  def clear_axisy
    @draw_data.axis_y.clear
    clear_draw
    @axisy_select.not_active
    @logybutton.set_active(false)
    @y_title.set_text("")
  end
  def clear_draw
    @drawbutton.set_sensitive(false)
    @dims.each_value{|dim| dim.select.not_active}
    @movie_box.set_sensitive(false)
    @movie_combo.entry.set_text("-")
  end

  def draw_x
    st = Array.new(@vars[@draw_data.var.name].dname.size,0)
    en = Array.new(@vars[@draw_data.var.name].dname.size,0)
    ax = 0
    ay = 0

    @vars[@draw_data.var.name].dname.each{|dim|
      index = @vars[@draw_data.var.name].dname.index(dim)
      if dim == @draw_data.axis_x.name then
	ax = index
	st[index]=0
	en[index]=-1
      elsif (@draw_data.type != LINE) && (dim == @draw_data.axis_y.name) then
	ay = index
	st[index]=0
	en[index]=-1
      else
	st[index] = @dims[dim].select.index
	en[index] = @dims[dim].select.index
	@draw_data.set_dim(dim,@dims[dim].select_value)
      end
    }

    var = @vars[@draw_data.var.name].vobj.get("start"=>st, "end"=>en)
    if (@draw_data.type != LINE) && (ax > ay) then
      var = var.transpose(1,0)
    end

    @draw_data.st = st
    @draw_data.en = en
    @draw_data.var.value = var

    @draw_data.axis_x.min = s2f(@x_min.get_text)
    @draw_data.axis_x.max = s2f(@x_max.get_text)
    @draw_data.axis_y.min = s2f(@y_min.get_text)
    @draw_data.axis_y.max = s2f(@y_max.get_text)

    if !@draw_data.axis_x.min then
      @draw_data.axis_x.min = @draw_data.axis_x.value[0]
    end
    if !@draw_data.axis_x.max then
      @draw_data.axis_x.max = @draw_data.axis_x.value[-1]
    end
    if !@draw_data.axis_y.min then
      if @draw_data.type==LINE
	@draw_data.axis_y.min = var.min
      else
	@draw_data.axis_y.min = @draw_data.axis_y.value[0]
      end
    end
    if !@draw_data.axis_y.max then
      if @draw_data.type==LINE
	@draw_data.axis_y.max = var.max
      else
	@draw_data.axis_y.max = @draw_data.axis_y.value[-1]
      end
    end

    if (min(@draw_data.axis_x.min,@draw_data.axis_x.max)<=0) and @draw_data.axis_x.log? then
      dialog = Gtk::Dialog.new.set_title("Error Message")
      dialog.set_modal(true)
      dialog.vbox.pack_start(Gtk::Label.new("X-Axis has negative value for log scale").show)
      ok_button=Gtk::Button.new("OK").show
      ok_button.signal_connect("clicked"){dialog.destroy}
      dialog.action_area.pack_start(ok_button)
      dialog.show
    elsif (min(@draw_data.axis_y.min,@draw_data.axis_y.max)<=0) and @draw_data.axis_y.log? then
      dialog = Gtk::Dialog.new.set_title("Error Message")
      dialog.set_modal(true)
      dialog.vbox.pack_start(Gtk::Label.new("Y-Axis has negative value for log scale").show)
      ok_button=Gtk::Button.new("OK").show
      ok_button.signal_connect("clicked"){dialog.destroy}
      dialog.action_area.pack_start(ok_button)
      dialog.show
    else

      @draw = Draw.new if @draw.nil?

      @draw_data.axis_x.title=@x_title.get_text
      @draw_data.axis_x.unit=@x_unit.get_text
      @draw_data.axis_y.title=@y_title.get_text
      @draw_data.axis_y.unit=@y_unit.get_text
      @draw_data.var.missing_value=@miss_entry.get_text
      @draw_data.title = @title_entry.get_text
      if @cont_level
	@draw_data.cont=[@cont_min_entry.get_text.to_f,@cont_interval_entry.get_text.to_f]
      else
	@draw_data.cont=nil
      end
      if @tone_level
	@draw_data.tone=[@tone_min_entry.get_text.to_f,@tone_interval_entry.get_text.to_f]
      else
	@draw_data.tone=nil
      end

      if @draw_data.type==MAP then
	p @map_combo.entry.get_text
	@draw_data.map_type=@maps.index(@map_combo.entry.get_text)+30
      end

      @draw.draw_x(@draw_data)
    end

  end

  def draw_next
    @dim_movie = @movie_combo.entry.get_text
    @dims[@dim_movie].select.next 
    draw_x
  end

# quit
  def file_close
    if @file then
      clear_var
      print "file close\n"
      @file.close
      @file = nil
      @vars.clear
      @dimtable.hide
      @dims.clear
    end
  end
  def finish
    file_close
    print "Bye\n"
    Gtk.main_quit
  end

end


$main = GDCL.new
# argv

if ARGV[0] != nil then
  $main.file_open(ARGV.shift)
end


# main loop
Gtk.main
