=begin header

  draw.rb - a part of main.rb

=end

require "gdk_imlib"

require "narray"
require "numru/dcl"
include NumRu


class Draw_data
  def initialize
    @var = Var.new
    @axis_x = Axis.new
    @axis_y = Axis.new
    @dims = Hash.new
    @levels = {"cont"=>nil,"tone"=>nil}
    clear
  end
  def var; @var; end
  def axis_x; @axis_x; end
  def axis_y; @axis_y; end
  def file_name=(str); @file_name=str; end
  def file_name; @file_name; end
  def st=(arr); @st=arr; end
  def st; @st; end
  def en=(arr); @en=arr; end
  def en; @en; end
  def title=(str); @title=str; end
  def title; @title; end
  def set_dim(str,value); @dims[str]=value; end
  def dims; @dims; end
  def type=(val); @type=val; end
  def type; @type; end
  def map_type=(val); @map_type=val; end
  def map_type; @map_type; end
  def cont=(cont); @levels["cont"]=cont; end
  def cont; @levels["cont"]; end
  def tone=(tone); @levels["tone"]=tone; end
  def tone; @levels["tone"]; end
  def clear
    @axis_x.clear
    @axis_y.clear
    @file_name = nil
    @var.clear
    @st = nil
    @en = nil
    @title = nil
    @dims.clear
    @type = LINE
    @map_type = nil
    @levels.each_key{|key| @levels[key]=nil}
  end
end

class Var
  def initialize
    clear
  end
  def name=(str); @name=str; end
  def name; @name; end
  def value=(val); @value=val; end
  def value; @value; end
  def missing_value=(value); @miss=value; end
  def missing_value; @miss; end
  def clear
    @name=nil
    @missing_value=nil
  end
end

class Axis
  def initialize
    clear
  end
  def name=(str); @name=str; end
  def name; @name; end
  def value=(narray); @value=narray; end
  def value; @value[@start..@end] if @name; end
  def ful_value; @value; end
  def min=(val); @min=val; end
  def min; @min; end
  def max=(val); @max=val; end
  def max; @max; end
  def title=(str); @title=str; end
  def title; @title; end
  def unit=(str); @unit=str; end
  def unit; @unit; end
  def log=(tf); @log=tf; end
  def log?; @log; end
  def clear
    @name=nil
    @value=nil
    @title=""
    @unit=""
    @start=0
    @end=-1
    @log=false
  end
end




class Draw
  def initialize
    @pixmap = nil
    @height=400
    @width=400

    @rect = Array.new(4)

    clear

  end


  def clear
    @type = nil
    @title = nil
    @vxmin=0.2; @vxmax=0.8
    @vymin=0.2; @vymax=0.8

    @miss=nil
    @x_title=""; @x_unit=""
    @y_title=""; @y_unit=""
    @fig = Fig.new
    @frame = Mouse_Events.new(@height,@width)

    @opened = false
  end


  def draw_x(draw_data)

    @filename = draw_data.file_name
    @varname = draw_data.var.name
    @axisxname = draw_data.axis_x.name
    @axisyname = draw_data.axis_y.name

    @xmin = draw_data.axis_x.min
    @xmax = draw_data.axis_x.max
    @ymin = draw_data.axis_y.min
    @ymax = draw_data.axis_y.max

    @var = draw_data.var.value

    @miss = draw_data.var.missing_value

    @title = draw_data.title
    @dims = draw_data.dims

    @axisx = draw_data.axis_x.value
    @logx = draw_data.axis_x.log?
    @axisy = draw_data.axis_y.value
    @logy = draw_data.axis_y.log?

    @x_title = draw_data.axis_x.title
    @x_unit = draw_data.axis_x.unit
    @y_title = draw_data.axis_y.title
    @y_unit = draw_data.axis_y.unit

    @type = draw_data.type
    @maptype = draw_data.map_type

    @cont = draw_data.cont
    @tone = draw_data.tone

    @fig.var_set(@filename,@varname,@axisxname,@axisyname,
		 draw_data.st,draw_data.en)

    @fig.title_set(@x_title,@x_unit,@y_title,@y_unit)


    if @window.nil?
      open_graphics
    end

    draw(@draw_area)
    @window.show
  end

  def open_graphics
    set_env

    @draw_area = Gtk::DrawingArea.new
    @draw_area.size(@height,@width)
    @draw_area.signal_connect("expose_event"){|w,p|
      expose_event(w,p)
    }
    @draw_area.signal_connect("configure_event"){|w,p|
      configure_event(w)
      @width =  w.allocation.width; @frame.width=@width
      @height = w.allocation.height; @frame.height=@height
    }
    @draw_area.signal_connect("button_press_event"){|w,p|
      if p.button==1 then
	px = (p.x-@frame.xoffset)/@frame.length
	py = 1 - (p.y-@frame.yoffset)/@frame.length
	print "press:",px,",",py,"\n"
	@frame.pressed(px,py,@vxmin,@vxmax,@vymin,@vymax)
      else
	w.get_toplevel.hide
      end
    }
    @draw_area.signal_connect("button_release_event"){|w,p|
      px = (p.x-@frame.xoffset)/@frame.length
      py = 1 - (p.y-@frame.yoffset)/@frame.length
      if @frame.move?
	@vxmin,@vxmax,@vymin,@vymax, = @frame.released(px,py)
	draw(w)
      end
    }
    @draw_area.signal_connect("motion_notify_event"){|w,p|
      px = (p.x-@frame.xoffset)/@frame.length
      py = 1 - (p.y-@frame.yoffset)/@frame.length
      if @frame.move?
	expose_event(w)
	@frame.draged(w,px,py)
      end
    }
    @draw_area.set_events(Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_MOTION_MASK|Gdk::BUTTON_RELEASE_MASK)
    @draw_area.show

    @window = Gtk::Window.new(Gtk::WINDOW_TOPLEVEL)
    @window.signal_connect("delete_event"){|w,p| w.hide; true}
    @window.signal_connect("destroy_event"){exit}
    @window.signal_connect("key_press_event"){|w,e|
      key = e.string
      p key+" key press"

      if key=="q" then
	w.hide
      elsif key=="\r" || key==" " then
	$main.draw_next
      end
    }
    @window.add(@draw_area)
    @window.set_title(@filename)
    @window.show

  end

  def save_code(filename)
    file = File.open(filename,"w")

    file.print(@fig.head+"\n\n")
    file.print(@fig.var+"\n\n")
    file.print(@fig.open+"\n\n")
    file.print(@fig.parm+"\n\n")
    file.print(@fig.window+"\n\n")
    file.print(@fig.viewport+"\n\n")
    file.print(@fig.frame+"\n\n")
    file.print(@fig.title+"\n\n")
    file.print(@fig.draw+"\n\n")
    file.print(@fig.close+"\n\n")

    file.close
  end

  def save_image(filename)
    geom = @pixmap.get_geometry
    mask = Gdk::Bitmap.new(@draw_area.window, geom[2], geom[3])
    im = GdkImlib::Image.create_from_drawable(@pixmap, mask, geom[0], geom[1], geom[2], geom[3])
    begin
      im.save(filename)
      p "save image to #{filename}"
    rescue
      dialog = Gtk::Dialog.new.set_title("Error Message")
      dialog.set_modal(true)
      image_type = File.basename(filename).sub(File.basename(filename,".*"),"")
      dialog.vbox.pack_start(Gtk::Label.new("Image type: "+image_type.upcase+" is not suported").show)
      ok_button=Gtk::Button.new("OK").show
      ok_button.signal_connect("clicked"){dialog.destroy}
      dialog.action_area.pack_start(ok_button)
      dialog.show
    end

  end

  private
  def set_env
    DCL::swlset("LWAIT", false )
    DCL::sglset("LCNTL", false )
    DCL::udlset("LMSG", false )
    @fig.parm_set('DCL::sglset("LCNTL", false )')
    @fig.parm_set('DCL::udlset("LMSG", false )')

    if @miss and @miss.to_f!=0 then
      DCL::gllset("LMISS", true )
      DCL::glrset("RMISS", @miss )
      @fig.parm_set('DCL::gllset("LMISS", true )')
      @fig.parm_set("DCL::glrset(\"RMISS\", #{@miss})")
    end
  end

  private
  def axis
    DCL::ussttl(@x_title, @x_unit, @y_title, @y_unit)
    DCL::usdaxs
    @fig.draw_set('DCL::ussttl(x_title, x_unit, y_title, y_unit)')
    @fig.draw_set('DCL::usdaxs')
    DCL::sglset("LCLIP", false )
    @fig.draw_set('DCL::sglset("LCLIP", false )')
    i=0
    @dims.each{|key,value|
      DCL::sgtxzv(@vxmax+0.01,@vymax-0.03*i,key+"="+value.to_s,0.02,0,-1,1)
      @fig.draw_set("i=#{i}")
      @fig.draw_set("DCL::sgtxzv(vxmax+0.01,vymax-0.03*i,\"#{key}=#{value.to_s}\",0.02,0,-1,1)")
      i+=1
    }
      
  end

  private
  def draw_1d(title)

    DCL::sgplzu(@axisx, @var, 1, 3 )
    @fig.draw_set('DCL::sgplzu(axisx, var, 1, 3 )')

    axis

    DCL::uxsttl("t", title, 0 )
    @fig.draw_set("title = \"#{title}\"")
    @fig.draw_set('DCL::uxsttl("t", title, 0 )')

  end

  private
  def draw_2d(title,type)

    if type==MAP then
      tmp = NArray.sfloat(@axisx.length+1)
      tmp[0..-2] = @axisx
      tmp[-1] = @axisx[-1] + (@axisx[-1]-@axisx[0])/(@axisx.length-1)
      @axisx = tmp
      @var.reshape!(@axisx.length-1,@axisy.length)
      tmp = NArray.sfloat(@axisx.length, @axisy.length)
      tmp[0..-2,true] = @var
      tmp[-1,true] = @var[0,true]
      @var = tmp
    end

    DCL::uwsgxa(@axisx)
    DCL::uwsgya(@axisy)
    @fig.draw_set('DCL::uwsgxa(axisx)')
    @fig.draw_set('DCL::uwsgya(axisy)')



    DCL::ueitlv
    @fig.draw_set('DCL::ueitlv')
    if @tone.nil?
      min = @var.min
      max = @var.max
      dm = 0
    else
      min,dm = @tone
      max = min+dm*(50-1)
    end
    if min!=max then
      DCL::uegtla(min, max, dm )
      @fig.draw_set("DCL::uegtla(#{min}, #{max}, #{dm} )")
    end
    DCL::uetonf(@var.reshape(@axisx.length,@axisy.length))
    @fig.draw_set('DCL::uetonf(var)')

    if @cont.nil?
      min = @var.min
      max = @var.max
      dm = 0
    else
      min,dm = @cont
      max = min+dm*(50-1)
    end
    if min!=max then
      DCL::udgcla(min, max, dm )
      @fig.draw_set("DCL::udgcla(#{min}, #{max}, #{dm} )")
    end
    DCL::udcntz(@var.reshape(@axisx.length,@axisy.length))
    @fig.draw_set('DCL::udcntz(var)')


    if type==CONTOUR then
      axis
      DCL::uzrset("ROFFXT", 0.06)
      DCL::uxsttl("t", title, 0 )
      @fig.draw_set('DCL::uzrset("ROFFXT", 0.06)')
      @fig.draw_set("title = \"#{title}\"")
      @fig.draw_set('DCL::uxsttl("t", title, 0 )')
    elsif type==MAP then
      DCL::umpglb
      @fig.draw_set("DCL::umpglb")
      DCL::sglset("LCLIP", false )
      @fig.draw_set('DCL::sglset("LCLIP", false )')
      DCL::sgtxzv((@vxmax+@vxmin)/2,@vymax+0.02,title,0.02,0.0,0,2)
      @fig.draw_set("title = \"#{title}\"")
      @fig.draw_set("DCL::sgtxzv((vxmax+vxmin)/2,vymax+0.02,title,0.02,0.0,0,2)")
      DCL::sglset("LCLIP", true )
      @fig.draw_set('DCL::sglset("LCLIP", true )')
    end

  end

  private
  def draw(drawing_area)
    @fig.window_set(@xmin,@xmax,@ymin,@ymax)

    @fig.viewport_set(@vxmin,@vxmax,@vymin,@vymax)


    @fig.frame_clear
    @fig.draw_clear

    if @type == MAP then
      itr = @maptype
      p itr
      DCL::grfrm
      @fig.frame_set("DCL::grfrm")
#      DCL::grswnd( @xmin, @xmax, @ymin, @ymax )
#      @fig.frame_set("DCL::grswnd(xmin,xmax,ymin,ymax)")
      DCL::grsvpt( @vxmin, @vxmax, @vymin, @vymax )
      @fig.frame_set("DCL::grsvpt(vxmin,vxmax,vymin,vymax)")
#      DCL::grssim((@vxmax-@vxmin)/2,0.0,0.0)
#      @fig.frame_set("DCL::grssim((vxmax-vxmin)/2,0.0,0.0)")
      DCL::grsmpl(0.0,90.0,0.0)
      @fig.frame_set("DCL::grsmpl(0.0,90.0,0.0)")
#      DCL::grstxy(-180,180,0,90)
#      @fig.frame_set("DCL::grstxy(xmin,xmax,ymin,ymax)")
      DCL::grstrn(itr)
      @fig.frame_set("DCL::grstrn(#{itr})")
      DCL::umpfit
      @fig.frame_set("DCL::umpfit")
      DCL::grstrf
      @fig.frame_set("DCL::grstrf")
    else
      itr = 1
      itr += 2 if @logx; itr += 1 if @logy
      DCL::grfrm
      @fig.frame_set("DCL::grfrm")
      DCL::grswnd( @xmin, @xmax, @ymin, @ymax )
      @fig.frame_set("DCL::grswnd(xmin,xmax,ymin,ymax)")
      DCL::grsvpt( @vxmin, @vxmax, @vymin, @vymax )
      @fig.frame_set("DCL::grsvpt(vxmin,vxmax,vymin,vymax)")
      DCL::grstrn(itr)
      @fig.frame_set("DCL::grstrn(#{itr})")
      DCL::grstrf
      @fig.frame_set("DCL::grstrf")
    end

    DCL::sglset("LCLIP", true )
    @fig.draw_set('DCL::sglset("LCLIP", true )')
    if @type == LINE then
      draw_1d(@title)
    elsif (@type==CONTOUR) || (@type==MAP) then
      draw_2d(@title,@type)
    end

    expose_event(drawing_area)

  end

  private
  def expose_event(widget,event=nil)
    if ! @pixmap.nil?
      gc = widget.style.fg_gc(widget.state)
      if event.nil?
	geom = widget.window.get_geometry
      else
	area = event.area
	geom = [area.x,area.y,area.width,area.height]
      end
      widget.window.draw_pixmap(gc, @pixmap, geom[0], geom[1],
				geom[0],geom[1], geom[2], geom[3])
    end
    false
  end

  private
  def configure_event(widget)
    geom = widget.window.get_geometry
    if (geom[2]>0 && geom[3]>0) then
      @pixmap = Gdk::Pixmap.new(widget.window, geom[2], geom[3], -1)
      @pixmap.draw_rectangle(widget.style.white_gc, true, 0, 0, 
			    geom[2], geom[3])
    end

    DCL::zgsdrw(widget)
    DCL::zgspmp(@pixmap)

    if !@opened then
      DCL::sgopn(4)
      @opened = true
    end

    true
  end





  private
  class Fig

    def initialize
      @head = ""
      @var = ""
      @open = ""
      @parm = Array.new
      @window = Array.new
      @viewport = Array.new
      @frame = Array.new
      @title = Array.new
      @draw = Array.new
      @close = ""

      @head = <<"END"
require "gtk"
require "narray"
require "numru/dcl"
require "numru/netcdf"

include NumRu
END

      @open = <<"END"
DCL::gropn(1)
END
      ymin=@ymin; ymax=@ymax
      vxmin=@vxmin; vxmax=@vxmax
      vymin=@vymin; vymax=@vymax
      @window[0] = "xmin = axisx[0]"
      @window[1] = "xmax = axisx[-1]"
      @window[2] = "ymin = 0.0"
      @window[3] = "ymax = 1.0"
      @viewport[0] = "vxmin = 0.2"
      @viewport[1] = "vxmax = 0.8"
      @viewport[2] = "vymin = 0.2"
      @viewport[3] = "vymax = 0.8"

      @title[0] = 'x_title = ""'
      @title[1] = 'x_unit = ""'
      @title[2] = 'y_title = ""'
      @title[3] = 'y_unit = ""'

      @close = <<"END"
DCL::grcls
END

    end

    def head
      @head
    end

    def var_set(filename,varname,axisxname,axisyname,st,en)
      @var = <<"END"
filename = "#{filename}"

file = NetCDF.open(filename,mode="r")

varname = "#{varname}"
varobj = file.var(varname)
st = [#{st.join(",")}]; en = [#{en.join(",")}]
var = varobj.get("start"=>st,"end"=>en)

axisxname = "#{axisxname}"
axisxobj = file.var(axisxname)
axisx = axisxobj.get

END
      if (axisyname!="") and (!axisyname.nil?) then
	@var << "axisyname = \"#{axisyname}\"\n"
	@var << "axisyobj = file.var(axisyname)\n"
	@var << "axisy = axisyobj.get\n"
      end

      @var << "\nfile.close\n"

    end
    def var
      @var
    end

    def open
      @open
    end

    def parm_set(str)
      @parm.push(str)
    end
    def parm
      @parm.join("\n")
    end

    def window_set(xmin,xmax,ymin,ymax)
      @window[0] = "xmin = #{xmin}"
      @window[1] = "xmax = #{xmax}"
      @window[2] = "ymin = #{ymin}"
      @window[3] = "ymax = #{ymax}"
    end
    def window
      @window.join("\n")
    end
  
    def viewport_set(vxmin,vxmax,vymin,vymax)
      @viewport[0] = "vxmin = #{vxmin}"
      @viewport[1] = "vxmax = #{vxmax}"
      @viewport[2] = "vymin = #{vymin}"
      @viewport[3] = "vymax = #{vymax}"
    end
    def viewport
      @viewport.join("\n")
    end

    def frame_clear
      @frame.clear
    end
    def frame_set(str)
      @frame.push(str)
    end
    def frame
      @frame.join("\n")
    end

    def title_set(xt,xu,yt,yu)
      @title[0] = "x_title = \"#{xt}\""
      @title[1] = "x_unit = \"#{xu}\""
      @title[2] = "y_title = \"#{yt}\""
      @title[3] = "y_unit = \"#{yu}\""
    end
    def title
      @title.join("\n")
    end


    def draw_clear
      @draw.clear
    end
    def draw_set(str)
      @draw.push(str)
    end
    def draw
      @draw.join("\n")
    end

    def close
      @close
    end

  end



end
